unit Strucdo1;

{
   Table Structure Documentor 2.2 - Main Unit
   Copyright (c) 1996 by Darryl G. Medley - All rights reserved.

   This software is distributed as freeware. It may be copied and used
   free of charge but MAY NOT BE SOLD by anyone other than the author,
   Darryl G. Medley.

   FeedBack:
   Compuserve ID: 103231,3036

   Solicitation:
   I am always looking for Delphi programming work. I have been a
   professional business applications programmer for over ten years and
   have extensive experience writing distribution and manufacturing
   systems. Please contact me via Compuserve if you are interested.
   Thanks.

   Credits:
   I have to give credit to Karl Thompson (Delphi Informant 3/96) for
   inspiration and the cool trick of using the "pos" function for finding
   the primary index fields. Also, Borland's TI # 2825, "Extracting Index
   Data From A Table" was helpful. Special thanks goes to Craig Rhinehart
   for help with testing & debugging on networks.

   Note:
   As I do not use dBASE tables, I am not handling complex dBASE indexes. To
   handle these, I think you have to test both IndexDefs.Items[i].Fields and
   IndexDefs.Items[i].Expression. TI 2825 says simple (1 field) indexes are in
   Fields and complex ones (multi-field or expression) are in Expression.
}

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, DB, DBTables, Printers,
  StrucHlp, Strucdir;

const
   left_marg = 5;   { left margin for printed output }

type
  TStrucDocMain = class(TForm)
    FldList: TListBox;
    Panel1: TPanel;
    BBtnOpen: TBitBtn;
    BBtnPrint: TBitBtn;
    BBtnHelp: TBitBtn;
    BBtnSaveText: TBitBtn;
    BBtnClose: TBitBtn;
    WorkTable: TTable;
    OpenDialog1: TOpenDialog;
    BBtnNext: TBitBtn;
    BBtnPrev: TBitBtn;
    BtnPrintAll: TBitBtn;
    BBtnSaveAll: TBitBtn;
    Bevel1: TBevel;
    SaveDialog1: TSaveDialog;
    PrintDialog1: TPrintDialog;
    procedure BBtnOpenClick(Sender: TObject);
    procedure BBtnCloseClick(Sender: TObject);
    procedure BBtnNextClick(Sender: TObject);
    procedure BBtnPrevClick(Sender: TObject);
    procedure BBtnPrintClick(Sender: TObject);
    procedure BBtnSaveTextClick(Sender: TObject);
    procedure BtnPrintAllClick(Sender: TObject);
    procedure BBtnHelpClick(Sender: TObject);
    procedure BBtnSaveAllClick(Sender: TObject);
  private
  public
    procedure LoadList;
    procedure OutputList(printing : boolean);
  end;

var
  StrucDocMain : TStrucDocMain;
  CurrTable : integer;
  OutFile : TFileName;
  SavePath : string;

implementation

{$R *.DFM}

{ =============== Utility Procedures & Functions =============== }

function ExtractFileName(const Path : string) : string;
         { remove path from file name }
var
   p : integer;
begin
   result := '';
   if path = '' then exit;

   p := length(Path);
   while (p > 0) and (not (path[p] in ['\',':'])) do
      dec(p);
   if p = 0 then
      result := UpperCase(Path)
   else
      result := UpperCase(copy(path,p+1,length(path)-p));
end;

function FixFileName(const Path : string) : string;
         { removes \\ from file name paths on MAP ROOT network drives }
var
   p : integer;
   work : string;
begin
   work := Path;
   repeat
      p := pos('\\',work);
      if p > 0 then delete(work,p,1);   { remove extra \ }
   until p = 0;
   result := work;
end;

function DTypeToStr(const DType : TFieldType) : string;
begin
  case DType of
     ftUnknown  : result := 'Unknown';
     ftString   : result := 'String';
     ftSmallInt : result := 'SmallInt';
     ftInteger  : result := 'Integer';
     ftWord     : result := 'Word';
     ftBoolean  : result := 'Boolean';
     ftFloat    : result := 'Float';
     ftCurrency : result := 'Currency';
     ftBCD      : result := 'BCD';
     ftDate     : result := 'Date';
     ftTime     : result := 'Time';
     ftDateTime : result := 'DateTime';
     ftBytes    : result := 'Bytes';
     ftVarBytes : result := 'VarBytes';
     ftBlob     : result := 'Blob';
     ftMemo     : result := 'Memo';
     ftGraphic  : result := 'Graphic';
  end;
end;   { DTypeToStr }

function KeyFlag(const FldName, PrimaryFlds : string) : string;
begin
   result := ' ';
   if pos(FldName,PrimaryFlds) > 0 then result := '*';
end;

function ReqToStr(const Req : boolean) : string;
begin
   result := '';
   if Req then result := 'Y';
end;

function SizeToStr(const Size : integer) : string;
begin
   result := '';
   if Size <> 0 then
      result := IntToStr(Size);
end;

procedure SplitIndexFlds(var LstItms : TListBox;
                         const LineTitle, Flds : string; const size : integer);
var
   work1, work2, tmptitle : string;
   i, l : integer;
begin
   work1 := Flds;
   l := length(LineTitle);
   tmptitle := LineTitle;
   while length(work1) > size do begin   { string longer than line }
      i := size;
      while (i > 0) and (work1[i] <> ';') do   { end of last complete field }
         dec(i);
      if i = 0 then i := size;   { trap bad string }
      LstItms.Items.Add(tmptitle + copy(work1,1,i));   { output string }
      tmptitle := copy('                    ',1,l);   { clear }
      delete(work1,1,i);
   end;
   if work1 <> '' then
      LstItms.Items.Add(tmptitle + work1);   { output remaining string }
end;   { SplitIndexFlds }

procedure TStrucDocMain.LoadList;
var
   i : integer;
   PrimaryFlds : string;
begin
   StrucDocMain.Caption := 'Structure of Table ' +
                           ExtractFileName(WorkTable.TableName);
   Screen.Cursor := crHourglass;
   try
      FldList.Clear;
      WorkTable.FieldDefs.Update;
      WorkTable.IndexDefs.Update;
      with FldList.Items, WorkTable do begin
         Add('Structure of Table ' +
                    ExtractFileName(WorkTable.TableName));
         Add('');
         Add(format('%3s %-25s %-8s %-4s %-3s %-3s',
                    ['#','Field Name','Type','Size','Key','Req']));
         { find primary index }
         i := 0;
         PrimaryFlds := '';
         while (i <= (IndexDefs.Count - 1)) and
            (not (ixPrimary in IndexDefs.Items[i].Options)) do
            inc(i);
         if i <= (IndexDefs.Count - 1) then
            PrimaryFlds := IndexDefs.Items[i].Fields;

         { load field list }
         for i := 0 to FieldDefs.Count - 1 do begin
            Add(format('%3d %-25s %-8s %4s  %s   %s',
                       [FieldDefs.Items[i].FieldNo,
                        FieldDefs.Items[i].Name,
                        DTypeToStr(FieldDefs.Items[i].DataType),
                        SizeToStr(FieldDefs.Items[i].Size),
                        KeyFlag(FieldDefs.Items[i].Name, PrimaryFlds),
                        ReqToStr(FieldDefs.Items[i].Required)]));
         end;   { field loop }

         { process indexes }
         if PrimaryFlds <> '' then begin
            Add('');
            Add('Primary Index Fields:');
            if length(PrimaryFlds) < 56 then
               Add('   ' + PrimaryFlds)
            else
               SplitIndexFlds(FldList,'   ',PrimaryFlds,56);
         end;
         if (IndexDefs.Count > 1) or
            ((IndexDefs.Count = 1) and
            (not (ixPrimary in IndexDefs.Items[0].Options)))
         then begin   { has secondary indexes }
            Add('');
            Add('Secondary Indexes:');
            for i := 1 to IndexDefs.Count - 1 do
               if not (ixPrimary in IndexDefs.Items[i].Options) then begin
                  Add('   Name  : ' + IndexDefs.Items[i].Name);
                  if length(IndexDefs.Items[i].Fields) < 48 then
                     Add('   Fields: ' + IndexDefs.Items[i].Fields)
                  else
                     SplitIndexFlds(FldList,'   Fields: ',
                                    IndexDefs.Items[i].Fields,48);
                  if i <> (IndexDefs.Count - 1) then { not last one }
                     Add('');   { blank line }
               end;
         end;   { if has secondary indexes }
      end;   { with FldList.Items }
   finally
      Screen.Cursor := crDefault;
   end;
end;   { LoadList }

procedure TStrucDocMain.OutputList(printing : boolean);
var
   out : TextFile;
   i, Page : integer;
begin
   Screen.Cursor := crHourglass;
   try
      if not printing then
         AssignFile(out, OutFile)
      else begin
         AssignPrn(out);
      end;
      Rewrite(out);
      if printing then begin
         printer.canvas.font.name := 'Courier New';
         printer.canvas.font.size := 10;
         printer.canvas.font.style := [fsBold];
         Page := printer.PageNumber;
         write(out,'':left_marg);
      end;
      write(out,FldList.Items[0]);   { print table name in bold }
      if printing then begin
         write(out,' on ',FormatDateTime('mm/dd/yy',date));
         printer.canvas.font.style := [];
      end;
      writeln(out);

      for i := 1 to FldList.Items.Count - 1 do begin   { main output loop }
         if printing then begin
            if (Page <> printer.PageNumber) then begin
               printer.canvas.font.style := [fsBold];
               Page := printer.PageNumber;
               write(out,'':left_marg);
               writeln(out,FldList.Items[0] + ' continued...');
               printer.canvas.font.style := [];
               writeln(out);
            end;
            write(out,'':left_marg);
         end;
         writeln(out,FldList.Items[i]);
      end;   { Main output loop }
      CloseFile(out);
   finally
      Screen.Cursor := crDefault;
   end;
end;   { OutputList }

{ =============== Event Handlers =============== }

procedure TStrucDocMain.BBtnOpenClick(Sender: TObject);
begin
   if OpenDialog1.Execute then begin
      BBtnPrint.Enabled := true;
      BBtnSaveText.Enabled := true;
      BBtnNext.Enabled := true;
      BBtnPrev.Enabled := true;
      BtnPrintAll.Enabled := true;
      BBtnSaveAll.Enabled := true;
      if OpenDialog1.Files.Count = 1 then begin
         BBtnNext.Hide;
         BBtnPrev.Hide;
         BtnPrintAll.Hide;
         BBtnSaveAll.Hide;
         Bevel1.Hide;
      end
      else begin
         BBtnNext.Show;
         BBtnPrev.Show;
         BtnPrintAll.Show;
         BBtnSaveAll.Show;
         Bevel1.Show;
         BBtnPrev.Enabled := false;
         BBtnNext.Enabled := true;
      end;
      CurrTable := 0;
      { display first file selected }
      WorkTable.TableName := FixFileName(OpenDialog1.Files[0]);
      LoadList;
   end
   else begin   { Open was cancelled - reset list & buttons }
      StrucDocMain.Caption := 'Table Structure Documentor 2.2';
      CurrTable := 0;
      BBtnPrint.Enabled := false;
      BBtnSaveText.Enabled := false;
      BBtnNext.Hide;
      BBtnPrev.Hide;
      BtnPrintAll.Hide;
      BBtnSaveAll.Hide;
      Bevel1.Hide;
      FldList.Clear;
   end;   { if OpenDialog1.Execute else }
end;

procedure TStrucDocMain.BBtnCloseClick(Sender: TObject);
begin
   Close;   { exit program }
end;

procedure TStrucDocMain.BBtnPrintClick(Sender: TObject);
begin
   if PrintDialog1.Execute then
      OutputList(true);
end;

procedure TStrucDocMain.BBtnSaveTextClick(Sender: TObject);
begin
   if SaveDialog1.Execute then begin
      OutFile := SaveDialog1.FileName;
      OutputList(false);
   end;
end;

procedure TStrucDocMain.BBtnHelpClick(Sender: TObject);
begin
   Application.CreateForm(THelpDlg,HelpDlg);
   HelpDlg.ShowModal;
   HelpDlg.Free;
end;

procedure TStrucDocMain.BBtnNextClick(Sender: TObject);
begin
   if CurrTable < (OpenDialog1.Files.Count - 1) then begin
      inc(CurrTable);
      if CurrTable = (OpenDialog1.Files.Count - 1) then
         BBtnNext.Enabled := false;
      if not BBtnPrev.Enabled then
         BBtnPrev.Enabled := true;
      WorkTable.TableName := FixFileName(OpenDialog1.Files[CurrTable]);
      LoadList;
   end;
end;

procedure TStrucDocMain.BBtnPrevClick(Sender: TObject);
begin
   if CurrTable > 0 then begin
      dec(CurrTable);
      if CurrTable = 0 then
         BBtnPrev.Enabled := false;
      if not BBtnNext.Enabled then
         BBtnNext.Enabled := true;
      WorkTable.TableName := FixFileName(OpenDialog1.Files[CurrTable]);
      LoadList;
   end;
end;

procedure TStrucDocMain.BtnPrintAllClick(Sender: TObject);
var
   i : integer;
begin
   if PrintDialog1.Execute then begin
      for i := 0 to OpenDialog1.Files.Count - 1 do begin
         WorkTable.TableName := FixFileName(OpenDialog1.Files[i]);
         LoadList;
         OutputList(true);
      end;
      WorkTable.TableName := FixFileName(OpenDialog1.Files[CurrTable]);
      LoadList;
   end;
end;

procedure TStrucDocMain.BBtnSaveAllClick(Sender: TObject);
var
   i, mr : integer;
begin
   Application.CreateForm(TSelDirDlg,SelDirDlg);
   mr := SelDirDlg.ShowModal;
   SelDirDlg.Free;
   if mr = mrOK then begin
      if SavePath[length(SavePath)] <> '\' then
         SavePath := SavePath + '\';
      for i := 0 to OpenDialog1.Files.Count - 1 do begin
         WorkTable.TableName := FixFileName(OpenDialog1.Files[i]);
         OutFile := ExtractFileName(WorkTable.TableName);
         if pos('.',OutFile) > 0 then
            delete(OutFile,pos('.',OutFile),3);   { remove extension }

         OutFile := SavePath + OutFile + '.TXT';
         LoadList;
         OutputList(false);
      end;
      WorkTable.TableName := FixFileName(OpenDialog1.Files[CurrTable]);
      LoadList;
   end;
end;

end.
